package com.xkrs.microservice.common.account;

import com.xkrs.microservice.common.encapsulation.PromptMessageEnum;
import com.xkrs.microservice.service.SysUserService;
import com.xkrs.microservice.util.IpUtil;
import com.xkrs.microservice.common.encapsulation.OutputEncapsulation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import javax.annotation.Resource;
import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.Locale;

/**
 * jwt登录过滤器
 * @author tajochen
 */
public class JwtLoginFilter extends AbstractAuthenticationProcessingFilter {

    private static final Logger logger = LoggerFactory.getLogger(JwtLoginFilter.class);


    @Resource
    private SysUserService sysUserService;

    public JwtLoginFilter(String url, AuthenticationManager authManager) {
        super(new AntPathRequestMatcher(url));
        setAuthenticationManager(authManager);
    }

    /**
     * 登录时验证
     * @param req 请求
     * @param res 响应
     */
    @Override
    public Authentication attemptAuthentication(HttpServletRequest req, HttpServletResponse res) {
        try {
            req.setCharacterEncoding("UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            logger.error(String.valueOf(e));
        }
        res.setHeader("Access-Control-Allow-Origin","*");
        res.setHeader("Access-Control-Allow-Credentials", "false");
        AccountCredentials cred = new AccountCredentials();
        //获取表单数据
        String userName = req.getParameter("userName");
        String password = req.getParameter("password");
        String rememberMe = req.getParameter("remember");
        //如果用户名密码为空
        if(userName == null||password == null|| userName.trim().isEmpty()||password.trim().isEmpty()){
            throw new BadCredentialsException("user or password is null");
        }
        if(rememberMe == null||rememberMe.isEmpty()){
            rememberMe = "false";
        }
        cred.setUserName(userName.trim());
        cred.setPassword(password.trim());
        cred.setRemember(Boolean.parseBoolean(rememberMe));
        // 返回一个验证令牌
        return getAuthenticationManager().authenticate(
            new UsernamePasswordAuthenticationToken(
                    cred.getUserName(),
                    cred.getPassword()
            )
        );
    }

    /**
     * 验证成功后调用
     * @param req 请求
     * @param response 响应
     * @param chain 过滤器声明
     * @param auth 认证
     */
    @Override
    protected void successfulAuthentication(
            HttpServletRequest req, HttpServletResponse response, FilterChain chain, Authentication auth) {
        if(sysUserService==null){
            ServletContext servletContext = req.getServletContext();
            WebApplicationContext webApplicationContext = WebApplicationContextUtils.getWebApplicationContext(servletContext);
            assert webApplicationContext != null;
            sysUserService = webApplicationContext.getBean(SysUserService.class);
        }
        //更新用户登录信息
        sysUserService.updateSysUserLogin(auth.getName(), IpUtil.getIpAddr(req));
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Credentials", "false");
        response.setContentType("application/json");
        response.setCharacterEncoding("UTF-8");
        TokenAuthenticationService.addAuthentication(response, auth.getName(),auth.getAuthorities());
    }

    /**
     * 验证失败后调用
     * @param request 请求
     * @param response 响应
     * @param failed 失败
     */
    @Override
    protected void unsuccessfulAuthentication(HttpServletRequest request,
                                              HttpServletResponse response,
                                              AuthenticationException failed) {
        Locale locale = LocaleContextHolder.getLocale();
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Credentials", "false");
        response.setContentType("application/json");
        response.setCharacterEncoding("UTF-8");
        // 将 JWT 写入 body
        PrintWriter out = null;
        try {
            out = response.getWriter();
        } catch (IOException e) {
            e.printStackTrace();
            logger.warn(String.valueOf(e));
        }
        assert out != null;
        out.append(OutputEncapsulation.outputEncapsulationObject(PromptMessageEnum.DATA_WRONG, failed.getLocalizedMessage(), locale));
        response.setStatus(HttpServletResponse.SC_OK);
    }
}
