package com.xkrs.microservice.common.tool;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.StandardCharsets;
import java.time.LocalDate;
import java.util.HashMap;
import java.util.Map;

import static com.xkrs.microservice.util.DateTimeUtil.dateToStringNoLine;

/**
 * 降水量
 * @author wd
 */
public class Precipitation {

    private static final Logger log = LoggerFactory.getLogger(Precipitation.class);

    /**
     * 历史降雨量
     */
    public static final String HISTORICAL_PRECIPITION_URL = "http://data.cma.cn/dataGis/multiSource/getLiveDataInfo3";

    /**
     * 预报天气
     */
    public static final String FORECAST_PERCIPITION_URL = "http://data.cma.cn/dataGis/exhibitionData/getStationInfo";

    private static final String USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 " +
            "(KHTML, like Gecko) Chrome/93.0.4577.63 Safari/537.36";

    private static final String COOKIE = "JSESSIONID=E0AF489776F9463892478C226957FB86; " +
            "pgv_pvi=4137831424; " +
            "userLoginKey=63730d21fa3139b077bae8a7801cf04e; " +
            "trueName=snnbnoone; " +
            "userName=54e05d733671bd38af26fc06640f3ba4; " +
            "PHPSESSID=eqs8ocriikbhmq28lt2lvdoft0; " +
            "pgv_si=s349517824; " +
            "Hm_lvt_d9508cf73ee2d3c3a3f628fe26bd31ab=1604459365,1604483274,1604547634,1605764770; " +
            "_pk_testcookie.6.dd70=1; " +
            "login_id_chat=0; floatDc=1; " +
            "Hm_lpvt_d9508cf73ee2d3c3a3f628fe26bd31ab=1606110326; " +
            "_pk_ref.6.dd70=%5B%22%22%2C%22%22%2C1606110326%2C%22https%3A%2F%2Fwww.baidu.com%2Flink%3Furl%3Dvc5Gc6810ZDAhZd1006vGN0aBUYxE0Hmp1CrfAJLz2G%26wd%3D%26eqid%3Debf41e300000ebb0000000025fa3742d%22%5D; " +
            "_pk_ses.6.dd70=1; _pk_ref.1.dd70=%5B%22%22%2C%22%22%2C1606110326%2C%22https%3A%2F%2Fwww.baidu.com%2Flink%3Furl%3Dvc5Gc6810ZDAhZd1006vGN0aBUYxE0Hmp1CrfAJLz2G%26wd%3D%26eqid%3Debf41e300000ebb0000000025fa3742d%22%5D; " +
            "_pk_id.1.dd70=e952ffa8190db13b.1603759181.13.1606110326.1606042123.; " +
            "_pk_ses.1.dd70=*; " +
            "login_name_chat=0; " +
            "_pk_ses.2.dd70=*; " +
            "_pk_id.6.dd70=156cb08aa1aab3cc.1603759181.15.1606110674.1606110326.; " +
            "_pk_id.2.dd70=3620e1ea6324ce18.1604461280.4.1606110674.1605837309.";

    public static final String REFERER = "http://data.cma.cn/dataGis/static/gridgis/";

    /**
     * 处理历史降水数据
     * @param params 参数列表
     * time:时间戳 20201127093100
     * needLSB:1
     * lat:经度 站点
     * lon:纬度 站点
     * dataType：CMPA_PRE
     * url：HISTORICAL_PRECIPITION_URL
     */
    public static String historyDataHandler(Map<String,String> params){
        // 数据
        String data = getWeatherData(params);

        // 建立treeJson
        try {
            ObjectMapper mapper = new ObjectMapper();
            JsonNode jsonNode = mapper.readTree(data);
            // 根据路径获取数据
            JsonNode dsNdoe = jsonNode.path("data");
            // 获取数据
            if (dsNdoe == null){
                log.info("无数据");
                return null;
            }
            for (int i = 0; i < dsNdoe.size(); i++) {
                JsonNode node = dsNdoe.get(i);
                if ("PRE_10MIN_24HOUR".equals(node.path("fastEle").asText())){
                    return node.path("value").asText();
                }
            }
        } catch (JsonProcessingException e) {
            log.info("转化Json失败{}", (Object) e.getStackTrace());
        }
        return null;
    }

    /**
     * 处理预报降水
     * @param params 参数列表
     * staId：站点ID
     * funitemmenuid：1150101020
     * typeCode：NWST
     * url：FORECAST_PERCIPITION_URL
     */
    public static Map<String,String> forecastDataHandler(Map<String,String> params){

        Map<String,String> map = new HashMap<>(3);

        // 获取日期
        LocalDate localDate = LocalDate.now();
        // 数据
        String data = getWeatherData(params);
        if (data.trim().isEmpty()){
            log.info("无数据");
            return null;
        }

        // 建立treeJson
        try {
            ObjectMapper mapper = new ObjectMapper();
            JsonNode jsonNode = mapper.readTree(data);
            // 根据路径获取数据
            JsonNode dsNdoe = jsonNode.path("list").get(0);
            final int rainDay = 3;
            // 获取数据
            for (int j = 0; j < rainDay; j++) {

                String time = dateToStringNoLine(localDate.plusDays(j));
                String time1 = dateToStringNoLine(localDate.plusDays(j + 1));

                float rain = 0;
                if (dsNdoe == null){
                    log.info("dsNode没有数据");
                    return null;
                }
                for (int i = 0; i < dsNdoe.size(); i++) {
                    JsonNode node = dsNdoe.get(i);
                    if (node.get(0).asText().trim().isEmpty() || node.get(0).asText().length() != 0){
                        if (node.get(0).asText().contains(time) || node.get(0).asText().contains(time1 + "000000")){
                            rain += Float.parseFloat(node.get(1).asText());
                        }
                    }
                }
                String rains = String.valueOf(rain);
                map.put(time,rains);
            }

            return map;
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 获取数据
     * @param params 数据列表
     */
    private static String getWeatherData(Map<String,String> params){
        int connTimes = 5;
        boolean isSuccess = false;

        String urlWithParam = urlWithParam(params);
        StringBuffer data = new StringBuffer();

        while(!isSuccess && connTimes > 0){
            try {
                URL url = new URL(urlWithParam);
                URLConnection conn = url.openConnection();


                InputStream is = conn.getInputStream();
                InputStreamReader isr = new InputStreamReader(is, StandardCharsets.UTF_8);
                BufferedReader br = new BufferedReader(isr);
                String line = null;

                while((line = br.readLine()) != null){
                    data.append(line);
                }
                String datas = data.toString();

                is.close();
                isr.close();
                br.close();
                if (datas.contains("503 Service Temporarily Unavailable")){
                    log.info("数据错误");
                    connTimes --;
                    try {
                        Thread.sleep(20000);
                    } catch (InterruptedException ex) {
                        log.info("睡眠错误{}", (Object) ex.getStackTrace());
                    }
                }
                isSuccess = true;
            } catch (IOException e) {
                log.info("降水连接超时");
                connTimes --;
                try {
                    Thread.sleep(20000);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
            }
        }

        return data.toString();
    }


    /**
     * 路径加上参数
     * @param params 参数列表
     */
    private static String urlWithParam(Map<String,String> params){
        final String url = "url";

        // 拼接字符串
        StringBuffer urlAddParam = new StringBuffer();
        urlAddParam.append(params.get("url"));
        urlAddParam.append("?");
        params.forEach((k,v) ->{
            if (!url.equals(k)){
                urlAddParam.append(k);
                urlAddParam.append("=");
                urlAddParam.append(v);
                urlAddParam.append("&");
            }
        });
        String urlWithParam = urlAddParam.toString();
        final String endWith = "&";
        if (urlWithParam.endsWith(endWith)){
            urlWithParam = urlWithParam.substring(0,urlWithParam.length()-1);
        }
        log.info(urlWithParam);
        return urlWithParam;
    }

    /**
     * 获取降水预报
     * @param params 参数列表
     *  staId：站点ID
     *  funitemmenuid：1150101020
     *  typeCode：NWST
     *  url：FORECAST_PERCIPITION_URL
     */
    public static String getRealTimeRain(Map<String,String> params){

        // 成功标志
        boolean isSuccess = false;
        // 尝试次数
        int connTimes = 10;

        // 创建响应对象
        CloseableHttpResponse response = null;
        String line = null;
        String sLine = null;

        String json = null;

        while(!isSuccess && connTimes > 0){
        // 创建http对象
        RequestConfig defaultRequestConfig = RequestConfig.custom()
                .setSocketTimeout(60000).setConnectTimeout(60000)
                .setConnectionRequestTimeout(60000)
                .build();
        CloseableHttpClient client = HttpClients.custom()
                .setDefaultRequestConfig(defaultRequestConfig).build();

        // 创建并设置URI
        URIBuilder uri = null;
        // 创建Get请求
        HttpGet get = null;
        String url = "";

        try {
            url = urlWithParam(params);
            uri = new URIBuilder(url);
            get = new HttpGet(uri.build());
            // 设置请求头
            get.setHeader("Accept"
                    , "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
            get.setHeader("Accept-Encoding"
                    , "gzip, deflate");
            get.setHeader("Accept-Language"
                    , "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7,zh-TW;q=0.6");
            get.setHeader("Connection"
                    , "keep-alive");
            get.setHeader("Cache-Control"
                    , "no-cache");
            get.setHeader("Host"
                    , "data.cma.cn");
            get.setHeader("Upgrade-Insecure-Requests"
                    , "1");
            get.setHeader("Cookie", COOKIE);
            get.setHeader("User-Agent", USER_AGENT);
            get.setHeader("Referer", REFERER);
        } catch (Exception e) {
            log.info("{}", (Object) e.getStackTrace());
        }

        //发送请求
        HttpEntity entity = null;
        InputStream is = null;
        BufferedReader br = null;
            try {
                response = client.execute(get);
                // 获取请求结果
                entity = response.getEntity();
                is = entity.getContent();
                br = new BufferedReader(
                        new InputStreamReader(is, StandardCharsets.UTF_8)
                );
                StringBuffer sb = new StringBuffer();
                while((line = br.readLine()) != null){
                    sb.append(line);
                    sb.append("/n");
                }
                sLine = sb.toString();
                if (sLine.contains("503 Service Temporarily Unavailable")){
                    log.info("数据出错");
                    connTimes --;
                    closeConn(is,br,response,client);
                    Thread.sleep(30000);
                    continue;
                }
                sLine = sLine.substring(sLine.indexOf("{"));
                sLine = sLine.substring(0,sLine.length()-2);
                if (sLine.contains("503 Service Temporarily Unavailable")){
                    return null;
                }
                ObjectMapper objectMapper = new ObjectMapper();
                //使用ObjectMapper对象对 User对象进行转换
                try {
                    json = objectMapper.writeValueAsString(sLine);
                } catch (JsonProcessingException e) {
                    log.info("json字符串转化异常{}", (Object) e.getStackTrace());
                }
                // ObjectMapper objectMapper = new ObjectMapper();
                isSuccess = true;
            } catch (ClientProtocolException e) {
                // TODO Auto-generated catch block
                log.info("请求超时等问题{}", (Object) e.getStackTrace());

            }catch (IOException e) {
                log.info("I/O问题{}", (Object) e.getStackTrace());
            } catch (InterruptedException e) {
                log.info("中断异常{}", (Object) e.getStackTrace());
            } finally {
                closeConn(is,br,response,client);
            }
        }
        return json;
    }

    /**
     * 关闭连接
     * @param is InputStream
     * @param br BufferedReader
     * @param response CloseableHttpResponse
     * @param client CloseableHttpClient
     */
    private static void closeConn(InputStream is,  BufferedReader br, CloseableHttpResponse response, CloseableHttpClient client ){
        try {
            if (null != is){
                is.close();
            }
            if (null != br){
                br.close();
            }
            if (null != response){
                response.close();
            }
            if (null != client){
                client.close();
            }

        } catch (IOException e) {
            log.info("IO异常{}", (Object) e.getStackTrace());
        }
    }

    public static void main(String[] args) {
        // 历史降水
        Map<String,String> hisParams = new HashMap<>(6);
        // 当前时间戳
        hisParams.put("time","20201127100400");
        // 站点经纬度
        hisParams.put("lat","36.9700");
        hisParams.put("lon","120.7300");
        hisParams.put("needLSB","1");
        hisParams.put("dataType","CMPA_PRE");
        hisParams.put("url",HISTORICAL_PRECIPITION_URL);

        historyDataHandler(hisParams);
        // 预报降水
        Map<String,String> foreParams = new HashMap<>(4);
        // 站点id
        foreParams.put("stationId","");
        foreParams.put("funitemmenuid","1150101020");
        foreParams.put("typeCode","NWST");
        foreParams.put("url",FORECAST_PERCIPITION_URL);
        forecastDataHandler(foreParams);

    }
}
