package com.xkrs.microservice.controller;

import com.xkrs.microservice.common.encapsulation.PromptMessageEnum;
import com.xkrs.microservice.model.entity.FileAppEntity;
import com.xkrs.microservice.model.qo.AppFileQo;
import com.xkrs.microservice.model.validation.AppFileQoInsert;
import com.xkrs.microservice.model.vo.FileServerResultVo;
import com.xkrs.microservice.service.AppManageService;
import com.xkrs.microservice.service.FileServerService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Locale;
import java.util.Optional;
import java.util.regex.Pattern;

import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationErrorList;
import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;
import static com.xkrs.microservice.common.tool.TokenUtil.getTokenUserName;

/**
 * App 管理服务
 * @author tajochen
 */
@RestController
@RequestMapping("/api/app-file")
public class AppManageController {

    /**
     * 获取区域信息
     */
    Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private AppManageService appManageService;

    @Resource
    private FileServerService fileServerService;

    @RequestMapping(value="/get/all",method = RequestMethod.GET)
    public String getAllAppFiles() {
        Iterable<FileAppEntity> list = appManageService.getAllFile();
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/new-version",method = RequestMethod.GET)
    @CrossOrigin
    public String appFileGetNew(@RequestParam("versionNumber") String versionNumber){
        String pattern = "^([1-9]\\d|[1-9])(.([1-9]\\d|\\d)){2}$";
        boolean isMatch = Pattern.matches(pattern, versionNumber);
        if(!isMatch){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"版本号错误",locale);
        }
        Iterable<FileAppEntity> list =  appManageService.checkLastVersionNumber(versionNumber);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/add",method = RequestMethod.POST)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_general_user','auth_administor')")
    @CrossOrigin
    public String appFileAdd(@Validated({AppFileQoInsert.class}) @RequestBody AppFileQo appFileQo,
            BindingResult bindingResult,@RequestHeader(value="Authorization") String token) {
        // 验证数据合法性
        if(bindingResult.hasErrors()){
            return outputEncapsulationErrorList(bindingResult.getFieldErrors(),locale);
        }
        // 检查版本号是否存在
        Iterable<FileAppEntity> appFileEntityIterable = appManageService.getByVersionNumber(appFileQo.getVersionNumber());
        for (FileAppEntity o : appFileEntityIterable) {
            if(o!=null){
                return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"重复的版本号",locale);
            }
        }
        // 获取当前用户名
        String userName = getTokenUserName(token);
        appManageService.add(appFileQo,userName);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }

    @RequestMapping(value="/delete",method = RequestMethod.DELETE)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
    @CrossOrigin
    public String appFileDelete(@RequestParam("id") Integer id) {
        Optional <FileAppEntity> sf = appManageService.getById(id);
        if(sf.isEmpty()){
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG,"指定id文件不存在",locale);
        }
        try {
            FileServerResultVo fileServerResultVo = fileServerService.deleteFile(sf.get().getMd5(),"");
            appManageService.delete(id);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,fileServerResultVo,locale);
        } catch (Exception e) {
            e.printStackTrace();
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"文件服务器出错",locale);
        }
    }
}
