package com.xkrs.microservice.controller;

import com.xkrs.microservice.common.encapsulation.PromptMessageEnum;
import com.xkrs.microservice.model.entity.CropClassifyRecordEntity;
import com.xkrs.microservice.model.qo.CropClassifyRecordQo;
import com.xkrs.microservice.model.validation.CropClassifyRecordQoInsert;
import com.xkrs.microservice.service.CropClassifyRecordService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Locale;
import java.util.Optional;

import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationErrorList;
import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;
import static com.xkrs.microservice.common.tool.TokenUtil.getTokenUserName;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;

/**
 * 作物种植记录数据服务
 * @author tajochen
 */
@RestController
@RequestMapping("/api/crop-classify-record")
public class CropClassifyRecordController {

    /**
     * 获取区域信息
     */
    Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private CropClassifyRecordService cropClassifyRecordService;

    @RequestMapping(value="/get/all",method = RequestMethod.GET)
    @CrossOrigin
    public String getCropClassifyAllRecord() {
        Iterable<CropClassifyRecordEntity> list = cropClassifyRecordService.getAllRecord();
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/admcode",method = RequestMethod.GET)
    @CrossOrigin
    public String getCropClassifyByAdmCode(@RequestParam("admCode") Integer admCode) {
        Iterable<CropClassifyRecordEntity> list = cropClassifyRecordService.getByAdmCode(admCode);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/towncode/date",method = RequestMethod.GET)
    @CrossOrigin
    public String getCropClassifyByTownCode(@RequestParam("townCode") String townCode,
                                           @RequestParam("startDateTs") Long startDateTs,
                                           @RequestParam("endDateTs") Long endDateTs) {
        if(startDateTs > endDateTs){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"",locale);
        }
        LocalDate sTime = timeMillisToDate(startDateTs);
        LocalDate eTime = timeMillisToDate(endDateTs);
        Iterable<CropClassifyRecordEntity> list = cropClassifyRecordService.getByTownCodeAndDate(townCode,sTime,eTime);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/admcode/date",method = RequestMethod.GET)
    @CrossOrigin
    public String getCropClassifyByAdmCodeAndDate(@RequestParam("admCode") Integer admCode,
                                                    @RequestParam("startDateTs") Long startDateTs,
                                                    @RequestParam("endDateTs") Long endDateTs) {
        if(startDateTs > endDateTs){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"",locale);
        }
        LocalDate sTime = timeMillisToDate(startDateTs);
        LocalDate eTime = timeMillisToDate(endDateTs);
        Iterable<CropClassifyRecordEntity> list = cropClassifyRecordService.getByAdmCodeAndDateTime(admCode,sTime,eTime);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/add",method = RequestMethod.POST)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_general_user','auth_administor')")
    @CrossOrigin
    public String cropClassifyAdd(@Validated({CropClassifyRecordQoInsert.class}) @RequestBody CropClassifyRecordQo cropClassifyRecordQo,
                                  BindingResult bindingResult, @RequestHeader(value="Authorization") String token) {
        // 验证数据合法性
        if(bindingResult.hasErrors()){
            return outputEncapsulationErrorList(bindingResult.getFieldErrors(),locale);
        }
        // 获取当前用户名
        String userName = getTokenUserName(token);
        cropClassifyRecordService.add(cropClassifyRecordQo,userName);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }

    @RequestMapping(value="/delete",method = RequestMethod.DELETE)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
    @CrossOrigin
    public String cropClassifyDelete(@RequestParam("id") Integer id) {
        Optional<CropClassifyRecordEntity> cropClassifyRecordEntity = cropClassifyRecordService.getById(id);
        if(cropClassifyRecordEntity.isEmpty()){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"参数错误",locale);
        }
        cropClassifyRecordService.delete(id);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }

}
