package com.xkrs.microservice.controller;

import com.xkrs.microservice.common.encapsulation.PromptMessageEnum;
import com.xkrs.microservice.model.entity.SysRoleEntity;
import com.xkrs.microservice.model.qo.SysUserQo;
import com.xkrs.microservice.model.validation.SysUserQoInsert;
import com.xkrs.microservice.model.vo.SysUserVo;
import com.xkrs.microservice.service.SysRoleService;
import com.xkrs.microservice.service.SysUserService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.util.List;
import java.util.Locale;

import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationErrorList;
import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;
import static com.xkrs.microservice.common.tool.TokenUtil.getTokenUserName;
import static com.xkrs.microservice.util.NumberUtil.isStrNumeric;

/**
 * 系统用户Controller
 * @author tajocehn
 */
@RestController
@RequestMapping(value = "/api/user")
public class SysUserController {

    /**
     * 获取区域信息
     */
    Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private SysUserService sysUserService;

    @Resource
    private SysRoleService sysRoleService;

    /**
     * 登录用户Token验证
     * @return
     */
    @RequestMapping(value = "/logged/check",method = RequestMethod.POST)
    public String loginUserTokenCheck(){
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }

    @RequestMapping(value = "/check/duplicate", method = RequestMethod.GET)
    public String checkDuplicate(@RequestParam(value="userName", required=false) String userName){
        // 验证用户名是否重复
        if(!sysUserService.checkUserName(userName)){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"OK",locale);
    }

    @RequestMapping(value = "/add", method = RequestMethod.POST)
    public String addUser(@Validated({SysUserQoInsert.class}) @RequestBody SysUserQo userQo,
                          BindingResult bindingResult){
        // 验证数据合法性
        if(bindingResult.hasErrors()){
            return outputEncapsulationErrorList(bindingResult.getFieldErrors(),locale);
        }
        // 验证用户名是否重复
        if(!sysUserService.checkUserName(userQo.getUserName())){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"",locale);
        }
        // 添加新用户
        sysUserService.addUser(userQo);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"OK",locale);
    }

    @RequestMapping(value="/get/all", method = RequestMethod.GET)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
    public String getAllSysUser(){
        Iterable<SysUserVo> sysUserDtoList = sysUserService.getAllSysUser();
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,sysUserDtoList,locale);
    }

    /**
     * 软删除指定id的普通用户
     * @param id
     * @return
     */
    @RequestMapping(value = "/general/delete", method = RequestMethod.DELETE)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_general_user')")
    public String deleteCustomUser(@RequestParam(value="userId", required=false) int id){
        // 验证数据合法性
        int res = sysUserService.softDeleteGeneralUser(id);
        if(res==1){
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL,"",locale);
        } else {
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
        }
    }

//    @RequestMapping(value = "/update", method = RequestMethod.POST)
//    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
//    public String updateUser(@Validated({SysUserQoUpdate.class}) @RequestBody SysUserQo userQo,
//                          BindingResult bindingResult){
//        // 验证数据合法性
//        if(bindingResult.hasErrors()){
//            return outputEncapsulationErrorList(bindingResult.getFieldErrors(),locale);
//        }
//        // 修改用户
//        sysUserService.updateSysUser(userQo);
//        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"OK",locale);
//    }

    @RequestMapping(value = "/update/portrait", method = RequestMethod.POST)
    public String updateUserPortrait(@RequestParam(value="portraitId") String portraitId,
                                     @RequestHeader(value="Authorization") String token){

        if(!isStrNumeric(portraitId)){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"",locale);
        }
        // 获取当前用户名
        String userName = getTokenUserName(token);
        // 头像是否正确
        int res = sysUserService.updateSysUserPortrait(userName, Integer.valueOf(portraitId));
        if(res==1){
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG,"",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"OK",locale);
    }

    @RequestMapping(value = "/update/password", method = RequestMethod.POST)
    public String updateUserPassword(@RequestParam(value="oldPassword") String oldPassword,
                                     @RequestParam(value="newPassword") String newPassword,
                                     @RequestHeader(value="Authorization") String token){
        // 获取当前用户名
        String userName = getTokenUserName(token);
        int res = sysUserService.updateSysUserPassword(userName,oldPassword,newPassword);
        final int r1 = 1;
        final int r2 = 2;
        final int r3 = 3;
        if(res==r1){
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG,"未知错误",locale);
        } else if(res==r2){
            return outputEncapsulationObject(PromptMessageEnum.DATA_WRONG,"旧密码错误",locale);
        } else if(res==r3){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"新密码不符合规则",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"OK",locale);
    }

    @RequestMapping(value = "/get/info", method = RequestMethod.GET)
    public String getUserInfo(@RequestHeader(value="Authorization") String token){
        // 获取当前用户名
        String userName = getTokenUserName(token);
        SysUserVo sysUserVo = sysUserService.getSysUserVoByUserName(userName);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,sysUserVo,locale);
    }

    @RequestMapping(value="/set/forbidden",method = RequestMethod.POST)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
    public String setUserForbidden(@RequestParam(value="userName", required=true) String userName) {
        String generalRole = "role_general_user";
        String farmerRole = "role_farm_user";
        if(sysUserService.checkUserName(userName)){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"userName 错误",locale);
        }
        List<SysRoleEntity> list1 = sysRoleService.getSysRoleListByUserName(userName);
        SysRoleEntity sysRoleEntity = list1.get(0);
        if(generalRole.equals(sysRoleEntity.getRoleName())  || farmerRole.equals(sysRoleEntity.getRoleName())){
            sysUserService.disableSysUser(userName);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.USER_NO_PERMISSION,"",locale);
    }

    @RequestMapping(value="/set/enable",method = RequestMethod.POST)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
    public String setUserEnable(@RequestParam(value="userName", required=true) String userName) {
        String generalRole = "role_general_user";
        String farmerRole = "role_farm_user";
        if(sysUserService.checkUserName(userName)){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"userName 错误",locale);
        }
        List<SysRoleEntity> list1 = sysRoleService.getSysRoleListByUserName(userName);
        SysRoleEntity sysRoleEntity = list1.get(0);
        if(generalRole.equals(sysRoleEntity.getRoleName())  || farmerRole.equals(sysRoleEntity.getRoleName())){
            sysUserService.enableSysUser(userName);
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.USER_NO_PERMISSION,"",locale);
    }

}
