package com.xkrs.microservice.controller;

import com.xkrs.microservice.common.encapsulation.PromptMessageEnum;
import com.xkrs.microservice.model.entity.TifProductEntity;
import com.xkrs.microservice.model.qo.TifProductQo;
import com.xkrs.microservice.model.validation.TifProductQoInsert;
import com.xkrs.microservice.service.TifProductService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Locale;
import java.util.Optional;

import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationErrorList;
import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;
import static com.xkrs.microservice.common.tool.TokenUtil.getTokenUserName;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;

/**
 * 栅格产品管理服务
 * @author tajochen
 */
@RestController
@RequestMapping("/api/tif-product")
public class TifProductController {

    /**
     * 获取区域信息
     */
    Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private TifProductService tifProductService;

    @RequestMapping(value="/get/all",method = RequestMethod.GET)
    public String getAllFiles() {
        Iterable<TifProductEntity> list = tifProductService.getAllFile();
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/product-code",method = RequestMethod.GET)
    public String getByProductCode(@RequestParam("productCode") String productCode) {
        TifProductEntity res = tifProductService.getByProductCode(productCode);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,res,locale);
    }

    @RequestMapping(value="/get/product-type",method = RequestMethod.GET)
    public String getByProductType(@RequestParam("productType") String productType) {
        Iterable<TifProductEntity> list = tifProductService.getByProductType(productType);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/admcode/product-type",method = RequestMethod.GET)
    public String getByAdmCodeAndProductType(@RequestParam("admCode") Integer admCode,
                                                    @RequestParam("productType") String productType) {
        Iterable<TifProductEntity> list = tifProductService.getByAdmCodeAndProductType(admCode,productType);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/admcode/product-type/date",method = RequestMethod.GET)
    public String getByAdmCodeAndProductTypeAndDate(@RequestParam("admCode") Integer admCode,
                                                    @RequestParam("productType") String productType,
                                      @RequestParam("startDateTs") Long startDateTs,
                                      @RequestParam("endDateTs") Long endDateTs) {
        if(startDateTs > endDateTs){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"",locale);
        }
        LocalDate sTime = timeMillisToDate(startDateTs);
        LocalDate eTime = timeMillisToDate(endDateTs);
        Iterable<TifProductEntity> list = tifProductService.getByAdmCodeAndProductTypeAndDate(admCode,productType,sTime,eTime);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }


    @RequestMapping(value="/add",method = RequestMethod.POST)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_general_user','auth_administor')")
    @CrossOrigin
    public String fileAdd(@Validated({TifProductQoInsert.class}) @RequestBody TifProductQo tifProductQo,
            BindingResult bindingResult,@RequestHeader(value="Authorization") String token) {
        // 验证数据合法性
        if(bindingResult.hasErrors()){
            return outputEncapsulationErrorList(bindingResult.getFieldErrors(),locale);
        }
        // 验证数据是否存在
        if(tifProductService.getByProductCode(tifProductQo.getProductCode())!=null){
            return outputEncapsulationObject(PromptMessageEnum.DATA_REPEAT,"数据已存在",locale);
        }
        // 获取当前用户名
        String userName = getTokenUserName(token);
        tifProductService.add(tifProductQo,userName);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }

    @RequestMapping(value="/delete",method = RequestMethod.DELETE)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
    @CrossOrigin
    public String fileDelete(@RequestParam("id") Integer id) {
        Optional<TifProductEntity> tifProductEntity = tifProductService.getById(id);
        if(tifProductEntity.isEmpty()){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"参数错误",locale);
        }
        tifProductService.delete(id);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }
}
