package com.xkrs.microservice.controller;

import com.xkrs.microservice.common.encapsulation.PromptMessageEnum;
import com.xkrs.microservice.model.entity.WcsRecordEntity;
import com.xkrs.microservice.model.qo.WcsRecordQo;
import com.xkrs.microservice.model.validation.WcsRecordQoInsert;
import com.xkrs.microservice.service.WcsRecordService;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Locale;
import java.util.Optional;

import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationErrorList;
import static com.xkrs.microservice.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;
import static com.xkrs.microservice.common.tool.TokenUtil.getTokenUserName;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;

/**
 * 栅格服务管理
 * @author tajochen
 */
@RestController
@RequestMapping("/api/wcs-record")
public class WcsRecordController {

    /**
     * 获取区域信息
     */
    Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private WcsRecordService wcsRecordService;

    @RequestMapping(value="/get/all",method = RequestMethod.GET)
    public String getAllFiles() {
        Iterable<WcsRecordEntity> list = wcsRecordService.getAllService();
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/service-code",method = RequestMethod.GET)
    public String getByProductCode(@RequestParam("serviceCode") String serviceCode) {
        WcsRecordEntity res = wcsRecordService.getByServiceCode(serviceCode);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,res,locale);
    }

    @RequestMapping(value="/get/product-type",method = RequestMethod.GET)
    public String getByProductType(@RequestParam("productType") String productType) {
        Iterable<WcsRecordEntity> list = wcsRecordService.getByProductType(productType);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }

    @RequestMapping(value="/get/admcode/product-type",method = RequestMethod.GET)
    public String getByAdmCodeAndProductTypeAndDate(@RequestParam("admCode") Integer admCode,
                                                    @RequestParam("productType") String productType) {
        Iterable<WcsRecordEntity> list = wcsRecordService.getByAdmCodeAndProductType(admCode,productType);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }


    @RequestMapping(value="/get/admcode/product-type/date",method = RequestMethod.GET)
    public String getByAdmCodeAndProductTypeAndDate(@RequestParam("admCode") Integer admCode,
                                                    @RequestParam("productType") String productType,
                                                    @RequestParam("startDateTs") Long startDateTs,
                                                    @RequestParam("endDateTs") Long endDateTs) {
        if(startDateTs > endDateTs){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"",locale);
        }
        LocalDate sTime = timeMillisToDate(startDateTs);
        LocalDate eTime = timeMillisToDate(endDateTs);
        Iterable<WcsRecordEntity> list = wcsRecordService.getByAdmCodeAndProductTypeAndDate(admCode,productType,sTime,eTime);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,list,locale);
    }


    @RequestMapping(value="/add",method = RequestMethod.POST)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_general_user','auth_administor')")
    @CrossOrigin
    public String fileAdd(@Validated({WcsRecordQoInsert.class}) @RequestBody WcsRecordQo wcsRecordQo,
            BindingResult bindingResult,@RequestHeader(value="Authorization") String token) {
        // 验证数据合法性
        if(bindingResult.hasErrors()){
            return outputEncapsulationErrorList(bindingResult.getFieldErrors(),locale);
        }
        // 验证数据是否存在
        if(wcsRecordService.getByServiceCode(wcsRecordQo.getServiceCode())!=null){
            return outputEncapsulationObject(PromptMessageEnum.DATA_REPEAT,"数据已存在",locale);
        }
        // 获取当前用户名
        String userName = getTokenUserName(token);
        wcsRecordService.add(wcsRecordQo,userName);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }

    @RequestMapping(value="/delete",method = RequestMethod.DELETE)
    @PreAuthorize("hasAnyAuthority('auth_system_manager','auth_administor')")
    @CrossOrigin
    public String fileDelete(@RequestParam("id") Integer id) {
        Optional<WcsRecordEntity> wcsRecordEntity = wcsRecordService.getById(id);
        if(wcsRecordEntity.isEmpty()){
            return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL,"参数错误",locale);
        }
        wcsRecordService.delete(id);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS,"",locale);
    }
}
