package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.AppFileDao;
import com.xkrs.microservice.model.entity.FileAppEntity;
import com.xkrs.microservice.model.qo.AppFileQo;
import com.xkrs.microservice.service.AppManageService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static com.xkrs.microservice.util.VersionNumberUtil.compareVersion;
import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;

/**
 * App管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "AppManageServiceCache")
public class AppManageServiceImpl implements AppManageService {

    Logger logger = LoggerFactory.getLogger(AppManageServiceImpl.class);

    @Resource
    private AppFileDao appFileDao;

    /**
     * 获取所有APP列表
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FileAppEntity> getAllFile(){
        return appFileDao.findAll(Sort.by(Sort.Direction.DESC, "addTime"));
    }

    /**
     * 获取指定id记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional getById(Integer id){
        return appFileDao.findById(id);
    }

    /**
     * 获取指定版本号的记录
     * @param versionNumber
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FileAppEntity> getByVersionNumber(String versionNumber){
        return appFileDao.findByVersionNumber(versionNumber);
    }

    /**
     * 检查 App 的版本号
     * @param versionNumber
     * @return
     */
    @Override
    public List<FileAppEntity> checkLastVersionNumber(String versionNumber){
        Iterable<FileAppEntity> res = appFileDao.findAll(Sort.by(Sort.Direction.DESC, "addTime"));
        List<FileAppEntity> res2 = new ArrayList<>();
        List<FileAppEntity> res3 = new ArrayList<>();
        // 获取新版本
        for(FileAppEntity s: res){
            // 比较版本号
            int n = compareVersion(versionNumber,s.getVersionNumber());
            if(n == -1){
                res2.add(s);
            }
        }
        if(res2.size()>0){
            String tempVersionNumber = res2.get(0).getVersionNumber();
            // 获取新版本
            for(FileAppEntity ss: res){
                // 比较版本号
                int n = compareVersion(tempVersionNumber,ss.getVersionNumber());
                if(n == -1){
                    tempVersionNumber = ss.getVersionNumber();
                }
            }
            // 获取最新新版本
            for(FileAppEntity ss: res){
                if(ss.getVersionNumber().equals(tempVersionNumber)){
                    res3.add(ss);
                }
            }
        }
        return res3;
    }

    /**
     * 新增APP记录
     * @param appFileQo
     * @param userName
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "AppManageServiceCache",allEntries = true)
    @Override
    public void add(AppFileQo appFileQo, String userName){
        FileAppEntity fileAppEntity = new FileAppEntity();
        CopyPropertiesUtil.copy( appFileQo, fileAppEntity);
        fileAppEntity.setAddUserName(userName);
        // 设置系统时间
        fileAppEntity.setAddTime(getNowTime());
        appFileDao.save(fileAppEntity);
    }

    /**
     * 删除APP记录
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "AppManageServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        appFileDao.deleteById(id);
    }

}
