package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.CropClassifyRecordDao;
import com.xkrs.microservice.model.entity.CropClassifyRecordEntity;
import com.xkrs.microservice.model.qo.CropClassifyRecordQo;
import com.xkrs.microservice.service.CropClassifyRecordService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import com.xkrs.microservice.util.IterUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;

/**
 * 种植分类记录数据服务管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "CropClassifyRecordServiceCache")
public class CropClassifyRecordServiceImpl implements CropClassifyRecordService {

    Logger logger = LoggerFactory.getLogger(CropClassifyRecordServiceImpl.class);

    @Resource
    private CropClassifyRecordDao cropClassifyRecordDao;

    /**
     * 获取所有种植分类记录
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<CropClassifyRecordEntity> getAllRecord(){
        return cropClassifyRecordDao.findAll();
    }

    /**
     * 获取指定id的种植分类记录
     * @param id 记录编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<CropClassifyRecordEntity> getById(Integer id){
        return cropClassifyRecordDao.findById(id);
    }

    /**
     * 获取指定行政区划的所有种植分类记录
     * @param admCode 行政区划代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<CropClassifyRecordEntity> getByAdmCode(Integer admCode){
        return cropClassifyRecordDao.findByCountyCode(admCode);
    }

    /**
     * 获取指定街道和日期的所有种植分类记录
     * @param townCode 街道代码
     * @param startDate 开始日期
     * @param endDate 结束日期
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<CropClassifyRecordEntity> getByTownCodeAndDate(String townCode, LocalDate startDate, LocalDate endDate){
        return cropClassifyRecordDao.findByTownCodeAndCropDateBetween(townCode,startDate,endDate);
    }

    /**
     * 获取指定行政区划和时间段的所有种植分类核查记录
     * @param admCode 行政区划代码
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<CropClassifyRecordEntity> getByAdmCodeAndDateTime(Integer admCode, LocalDate startDate, LocalDate endDate){
        return cropClassifyRecordDao.findByCountyCodeAndCropDateBetween(admCode,startDate,endDate);
    }

    /**
     * 新增种植分类记录
     * @param cropClassifyRecordQo
     * @param userName 用户名
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "CropClassifyRecordServiceCache",allEntries = true)
    @Override
    public void add(CropClassifyRecordQo cropClassifyRecordQo, String userName){
        CropClassifyRecordEntity cropClassifyRecordEntity = new CropClassifyRecordEntity();
        CopyPropertiesUtil.copy(cropClassifyRecordQo, cropClassifyRecordEntity);
        LocalDate cropDate = timeMillisToDate(cropClassifyRecordQo.getCropDateTs());
        // 判断数据是否存在
        Iterable<CropClassifyRecordEntity> ss = cropClassifyRecordDao.findByTownCodeAndCropCodeAndCropDateBetween(cropClassifyRecordQo.getTownCode(),cropClassifyRecordQo.getCropCode(),cropDate,cropDate);
        // 若存在则覆盖
        if(IterUtil.isNotEmpty(ss)){
            CropClassifyRecordEntity old = ss.iterator().next();
            cropClassifyRecordEntity.setId(old.getId());
        }
        cropClassifyRecordEntity.setCropDate(cropDate);
        // 设置添加用户
        cropClassifyRecordEntity.setAddUserName(userName);
        // 设置系统时间
        cropClassifyRecordEntity.setAddTime(getNowTime());
        cropClassifyRecordDao.save(cropClassifyRecordEntity);
    }

    /**
     * 删除种植分类记录
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "CropClassifyRecordServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        cropClassifyRecordDao.deleteById(id);
    }

}
