package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.FarmingInfoDao;
import com.xkrs.microservice.model.entity.FarmingInfoEntity;
import com.xkrs.microservice.model.qo.FarmingInfoQo;
import com.xkrs.microservice.model.qo.TypicalFarmerQo;
import com.xkrs.microservice.service.FarmingInfoService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.*;

/**
 * FarmingInfoService 农情上报接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "FarmingInfoServiceCache")
public class FarmingInfoServiceImpl implements FarmingInfoService {

    Logger logger = LoggerFactory.getLogger(FarmingInfoServiceImpl.class);

    @Resource
    private FarmingInfoDao farmingInfoDao;

    /**
     * 获取所有农情上报记录列表
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FarmingInfoEntity> getAllRecord(){
        return farmingInfoDao.findAll(Sort.by(Sort.Direction.DESC, "createTime"));
    }

    /**
     * 获取指定id农情上报记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional getById(Integer id){
        return farmingInfoDao.findById(id);
    }

    /**
     * 获取指定乡镇代码的所有农情上报记录列表
     * @param dkbh 地块编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FarmingInfoEntity> getByDkbh(String dkbh){
        return farmingInfoDao.findAllByDkbh(dkbh);
    }

    /**
     * 查找农情信息根据区县编号
     * @param qxbh 区县编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FarmingInfoEntity> getByQxbh(Integer qxbh){
        return farmingInfoDao.findAllByQxbh(qxbh);
    }

    /**
     * 查找农情信息根据作物代码，区县编号
     * @param qxbh 区县编号
     * @param zwbh 作物代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FarmingInfoEntity> getByZwdmAndQxbh(Integer zwbh,Integer qxbh){
        return farmingInfoDao.findAllByZwdmAndQxbh(zwbh, qxbh);
    }

    /**
     * 获取所有新农情根据区县编号
     * @param qxbh
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FarmingInfoEntity> getNewByQxbh(Integer qxbh){
        return farmingInfoDao.findNewAllByQxbh(qxbh);
    }

    /**
     * 获取指定用户名的农情上报记录列表
     * @param userName 用户名
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<FarmingInfoEntity> getByUserName(String userName){
        return farmingInfoDao.findAllByAddUserName(userName);
    }

    /**
     * 新增农情上报记录
     * @param farmingInfoQo
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "FarmingInfoServiceCache",allEntries = true)
    @Override
    public void add(FarmingInfoQo farmingInfoQo, String userName){
        FarmingInfoEntity farmingInfoEntity = new FarmingInfoEntity();
        CopyPropertiesUtil.copy( farmingInfoQo, farmingInfoEntity);
        farmingInfoEntity.setAddUserName(userName);
        // 设置系统时间
        farmingInfoEntity.setCreateTime(getNowTime());
        farmingInfoDao.save(farmingInfoEntity);
    }

    /**
     * 删除农情上报记录
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "FarmingInfoServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        farmingInfoDao.deleteById(id);
    }

}
