package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.HydrologicalStationDao;
import com.xkrs.microservice.model.entity.HydrologicalStationEntity;
import com.xkrs.microservice.model.qo.HydrologicalStationQo;
import com.xkrs.microservice.service.HydrologicalStationService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import com.xkrs.microservice.util.IterUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.*;

/**
 * 水文站点服务接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "HydrologicalStationServiceCache")
public class HydrologicalStationServiceImpl implements HydrologicalStationService {

    Logger logger = LoggerFactory.getLogger(HydrologicalStationServiceImpl.class);

    @Resource
    private HydrologicalStationDao hydrologicalStationDao;

    /**
     * 获取所有水文站点数据
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<HydrologicalStationEntity> getAllRecord(){
        return hydrologicalStationDao.findAll();
    }

    /**
     * 获取时间段水文站点数据
     * @param sTime 开始时间
     * @param eTime 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<HydrologicalStationEntity> getByTime(LocalDateTime sTime,LocalDateTime eTime){
        return hydrologicalStationDao.findByRecordTimeBetween(sTime,eTime);
    }

    /**
     * 新增水文站点数据
     * @param hydrologicalStationQo 水文站点数据
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "HydrologicalStationServiceCache",allEntries = true)
    @Override
    public void add(HydrologicalStationQo hydrologicalStationQo){
        HydrologicalStationEntity hydrologicalStationEntity = new HydrologicalStationEntity();
        CopyPropertiesUtil.copy( hydrologicalStationQo, hydrologicalStationEntity);
        hydrologicalStationEntity.setCountyName(hydrologicalStationQo.getCounty());
        LocalDateTime theTime = timeMillisToTime(hydrologicalStationQo.getTime());
        // 判断数据是否存在
        LocalDateTime todayStart = LocalDateTime.of(theTime.toLocalDate(), LocalTime.MIN);
        LocalDateTime todayEnd = LocalDateTime.of(theTime.toLocalDate(), LocalTime.MAX);
        Iterable<HydrologicalStationEntity> ss = hydrologicalStationDao.findByNameAndRecordTimeBetween(hydrologicalStationQo.getName(),todayStart,todayEnd);
        // 若存在则跳过
        if(IterUtil.isNotEmpty(ss)){
            return;
        }
        // 如果
        hydrologicalStationEntity.setRecordTime(theTime);
        hydrologicalStationEntity.setCreateTime(getNowTime());
        hydrologicalStationDao.save(hydrologicalStationEntity);
    }

    /**
     * 删除水文站点数据
     * @param id 编号
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "HydrologicalStationServiceCache",allEntries = true)
    @Override
    public int delete(Integer id){
        Optional<HydrologicalStationEntity> res = hydrologicalStationDao.findById(id);
        if(res.isPresent()){
            hydrologicalStationDao.deleteById(id);
            return 0;
        } else {
            return 1;
        }
    }

}
