package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.IndependentCheckDao;
import com.xkrs.microservice.model.entity.IndependentCheckEntity;
import com.xkrs.microservice.model.qo.IndependentCheckQo;
import com.xkrs.microservice.service.IndependentCheckService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;
import static com.xkrs.microservice.util.OpenGeoUtil.wktStrToGeom;

/**
 * FarmingInfoService 自主核查上报接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "IndependentCheckServiceCache")
public class IndependentCheckServiceImpl implements IndependentCheckService {

    Logger logger = LoggerFactory.getLogger(IndependentCheckServiceImpl.class);

    @Resource
    private IndependentCheckDao independentCheckDao;

    /**
     * 获取所有自主核查记录列表
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IndependentCheckEntity> getAllRecord(){
        return independentCheckDao.findAll(Sort.by(Sort.Direction.DESC, "createTime"));
    }

    /**
     * 获取指定id自主核查记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional getById(Integer id){
        return independentCheckDao.findById(id);
    }

    /**
     * 获取指定区县代码的自主核查记录
     * @param countyCode 区县代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IndependentCheckEntity> getByCountyCode(Integer countyCode){
        return independentCheckDao.findByQxdm(countyCode);
    }

    /**
     * 获取指定用户名的自主核查记录列表
     * @param userName 用户名
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IndependentCheckEntity> getByUserName(String userName){
        return independentCheckDao.findAllByAddUserName(userName);
    }

    /**
     * 新增自主核查记录
     * @param independentCheckQo
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "IndependentCheckServiceCache",allEntries = true)
    @Override
    public void add(IndependentCheckQo independentCheckQo, String userName){
        IndependentCheckEntity independentCheckEntity = new IndependentCheckEntity();
        CopyPropertiesUtil.copy( independentCheckQo, independentCheckEntity);
        independentCheckEntity.setAddUserName(userName);
        // 转化geometry
        independentCheckEntity.setGeom(wktStrToGeom(independentCheckQo.getGeomStr()));
        // 设置系统时间
        independentCheckEntity.setCreateTime(getNowTime());
        independentCheckDao.save(independentCheckEntity);
    }

    /**
     * 删除自主核查记录
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "IndependentCheckServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        independentCheckDao.deleteById(id);
    }

}
