package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.IrrigationCheckDao;
import com.xkrs.microservice.dao.TypicalPlotIrrigationDao;
import com.xkrs.microservice.model.entity.IrrigationCheckEntity;
import com.xkrs.microservice.model.entity.TypicalPlotIrrigationEntity;
import com.xkrs.microservice.model.qo.IrrigationCheckQo;
import com.xkrs.microservice.service.IrrigationCheckService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.cache.annotation.Caching;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;

/**
 * FarmingInfoService 灌溉核查上报接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "IrrigationCheckServiceCache")
public class IrrigationCheckServiceImpl implements IrrigationCheckService {

    Logger logger = LoggerFactory.getLogger(IrrigationCheckServiceImpl.class);

    @Resource
    private IrrigationCheckDao irrigationCheckDao;

    @Resource
    private TypicalPlotIrrigationDao typicalPlotIrrigationDao;

    /**
     * 获取所有灌溉核查上报记录列表
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationCheckEntity> getAllRecord(){
        return irrigationCheckDao.findAll(Sort.by(Sort.Direction.DESC, "createTime"));
    }

    /**
     * 获取指定id灌溉核查上报记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional getById(Integer id){
        return irrigationCheckDao.findById(id);
    }


    /**
     * 获取指定用户的所有灌溉核查记录
     * @param userName 用户名
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationCheckEntity> getByUserName(String userName){
        return irrigationCheckDao.findByAddUserName(userName);
    }

    /**
     * 获取指定地块编号的所有灌溉核查记录
     * @param dkbh 地块编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationCheckEntity> getByDkbh(String dkbh){
        return irrigationCheckDao.findAllByDkbh(dkbh);
    }

    /**
     * 获取指定地块编号的所有区县编号
     * @param qxbh 区县编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationCheckEntity> getByQxbh(Integer qxbh){
        return irrigationCheckDao.findAllByQxbh(qxbh);
    }

    /**
     * 查找灌溉核查结果根据区县编号和时间
     * @param qxbh 区县编号
     * @param sTime 开始时间
     * @param eTime 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationCheckEntity> getByQxbhAndCreateTimeBetween(Integer qxbh, LocalDateTime sTime, LocalDateTime eTime){
        return irrigationCheckDao.findAllByQxbhAndCreateTimeBetween(qxbh,sTime,eTime);
    }

    /**
     * 获取指定灌溉编号的所有灌溉核查记录
     * @param ggbh 灌溉编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<IrrigationCheckEntity> getByGgbh(String ggbh){
        return irrigationCheckDao.findByGgbh(ggbh);
    }

    /**
     * 新增灌溉核查上报
     * @param irrigationCheckQo
     */
    @Transactional(rollbackFor=Exception.class)
    @Caching(evict={@CacheEvict(value = "IrrigationCheckServiceCache",allEntries = true),
            @CacheEvict(value = "TypicalPlotIrrigationService",allEntries = true)})
    @Override
    public void add(IrrigationCheckQo irrigationCheckQo, String userName){
        IrrigationCheckEntity irrigationCheckEntity = new IrrigationCheckEntity();
        CopyPropertiesUtil.copy( irrigationCheckQo, irrigationCheckEntity);
        irrigationCheckEntity.setAddUserName(userName);
        // 设置系统时间
        irrigationCheckEntity.setCreateTime(getNowTime());
        irrigationCheckDao.save(irrigationCheckEntity);
        // 修改灌溉记录
        Optional<TypicalPlotIrrigationEntity> typicalPlotIrrigationEntityOptional = typicalPlotIrrigationDao.findByGgbh(irrigationCheckEntity.getGgbh());
        if (typicalPlotIrrigationEntityOptional.isPresent()){
            TypicalPlotIrrigationEntity typicalPlotIrrigationEntity = typicalPlotIrrigationEntityOptional.get();
            typicalPlotIrrigationEntity.setHsbj(2);
            typicalPlotIrrigationEntity.setQrbj(irrigationCheckEntity.getHszt());
            typicalPlotIrrigationDao.save(typicalPlotIrrigationEntity);
        }
    }

    /**
     * 删除灌溉核查上报
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "IrrigationCheckServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        irrigationCheckDao.deleteById(id);
    }

}
