package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.IrrigationRecordDao;
import com.xkrs.microservice.model.entity.IrrigationRecordEntity;
import com.xkrs.microservice.model.qo.IrrigationRecordQo;
import com.xkrs.microservice.service.IrrigationRecordService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import com.xkrs.microservice.util.IterUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.*;

/**
 * 灌溉记录数据服务管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "IrrigationRecordServiceCache")
public class IrrigationRecordServiceImpl implements IrrigationRecordService {

    Logger logger = LoggerFactory.getLogger(IrrigationRecordServiceImpl.class);

    @Resource
    private IrrigationRecordDao irrigationRecordDao;

    /**
     * 获取所有灌溉记录
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationRecordEntity> getAllRecord(){
        return irrigationRecordDao.findAll();
    }

    /**
     * 获取指定id的灌溉记录
     * @param id 记录编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<IrrigationRecordEntity> getById(Integer id){
        return irrigationRecordDao.findById(id);
    }

    /**
     * 获取指定行政区划的所有灌溉记录
     * @param admCode 行政区划代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationRecordEntity> getByAdmCode(Integer admCode){
        return irrigationRecordDao.findByCountyCode(admCode);
    }

    /**
     * 获取指定街道代码和时间段的所有灌溉记录
     * @param townCode 街道代码
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationRecordEntity> getByTownCodeAndDate(String townCode, LocalDate startDate, LocalDate endDate){
        return irrigationRecordDao.findByTownCodeAndIrrigationDateBetween(townCode, startDate, endDate);
    }

    /**
     * 获取指定行政区划和时间段的所有灌溉记录
     * @param admCode 行政区划代码
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationRecordEntity> getByAdmCodeAndDateTime(Integer admCode, LocalDate startDate, LocalDate endDate){
        return irrigationRecordDao.findByCountyCodeAndIrrigationDateBetween(admCode,startDate,endDate);
    }

    /**
     * 新增灌溉记录
     * @param irrigationFieldCheckQo
     * @param userName 用户名
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "IrrigationRecordServiceCache",allEntries = true)
    @Override
    public void add(IrrigationRecordQo irrigationFieldCheckQo, String userName){
        IrrigationRecordEntity irrigationRecordEntity = new IrrigationRecordEntity();
        CopyPropertiesUtil.copy(irrigationFieldCheckQo, irrigationRecordEntity);
        LocalDate irrigationDate = timeMillisToDate(irrigationFieldCheckQo.getIrrigationDateTs());
        // 判断数据是否存在
        Iterable<IrrigationRecordEntity> ss = irrigationRecordDao.findByTownCodeAndIrrigationDateBetween(irrigationFieldCheckQo.getTownCode(),irrigationDate,irrigationDate);
        // 若存在则覆盖
        if(IterUtil.isNotEmpty(ss)){
            IrrigationRecordEntity old = ss.iterator().next();
            irrigationRecordEntity.setId(old.getId());
        }
        irrigationRecordEntity.setIrrigationDate(irrigationDate);
        // 设置添加用户
        irrigationRecordEntity.setAddUserName(userName);
        // 设置系统时间
        irrigationRecordEntity.setAddTime(getNowTime());
        irrigationRecordDao.save(irrigationRecordEntity);
    }

    /**
     * 删除灌溉记录
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "IrrigationRecordServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        irrigationRecordDao.deleteById(id);
    }

}
