package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.IrrigationSarRecordDao;
import com.xkrs.microservice.model.entity.IrrigationSarRecordEntity;
import com.xkrs.microservice.model.qo.IrrigationSarRecordQo;
import com.xkrs.microservice.service.IrrigationSarRecordService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import com.xkrs.microservice.util.IterUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;

/**
 * 灌溉Sar记录数据服务管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "IrrigationSarRecordServiceCache")
public class IrrigationSarRecordServiceImpl implements IrrigationSarRecordService {

    Logger logger = LoggerFactory.getLogger(IrrigationSarRecordServiceImpl.class);

    @Resource
    private IrrigationSarRecordDao irrigationSarRecordDao;

    /**
     * 获取所有灌溉记录
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationSarRecordEntity> getAllRecord(){
        return irrigationSarRecordDao.findAll();
    }

    /**
     * 获取指定id的灌溉记录
     * @param id 记录编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<IrrigationSarRecordEntity> getById(Integer id){
        return irrigationSarRecordDao.findById(id);
    }

    /**
     * 获取指定行政区划的所有灌溉记录
     * @param admCode 行政区划代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationSarRecordEntity> getByAdmCode(Integer admCode){
        return irrigationSarRecordDao.findByCountyCode(admCode);
    }

    /**
     * 获取指定街道代码和时间段的所有灌溉记录
     * @param townCode 街道代码
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationSarRecordEntity> getByTownCodeAndDate(String townCode, LocalDate startDate, LocalDate endDate){
        return irrigationSarRecordDao.findByTownCodeAndIrrigationDateBetween(townCode, startDate, endDate);
    }

    /**
     * 获取灌溉统计动态多条件查询
     * @param townCode 行政区划代码
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationSarRecordEntity> geAllByCondition(String townCode, LocalDate startDate, LocalDate endDate){
        return irrigationSarRecordDao.getAllByCondition(townCode,startDate,endDate);
    }

    /**
     * 获取指定行政区划和时间段的所有灌溉记录
     * @param admCode 行政区划代码
     * @param startDate 开始时间
     * @param endDate 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<IrrigationSarRecordEntity> getByAdmCodeAndDateTime(Integer admCode, LocalDate startDate, LocalDate endDate){
        return irrigationSarRecordDao.findByCountyCodeAndIrrigationDateBetween(admCode,startDate,endDate);
    }

    /**
     * 新增灌溉记录
     * @param irrigationFieldCheckQo
     * @param userName 用户名
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "IrrigationSarRecordServiceCache",allEntries = true)
    @Override
    public void add(IrrigationSarRecordQo irrigationFieldCheckQo, String userName){
        IrrigationSarRecordEntity irrigationSarRecordEntity = new IrrigationSarRecordEntity();
        CopyPropertiesUtil.copy(irrigationFieldCheckQo, irrigationSarRecordEntity);
        LocalDate irrigationDate = timeMillisToDate(irrigationFieldCheckQo.getIrrigationDateTs());
        // 判断数据是否存在
        Iterable<IrrigationSarRecordEntity> ss = irrigationSarRecordDao.findByTownCodeAndIrrigationDateBetween(irrigationFieldCheckQo.getTownCode(),irrigationDate,irrigationDate);
        // 若存在则覆盖
        if(IterUtil.isNotEmpty(ss)){
            IrrigationSarRecordEntity old = ss.iterator().next();
            irrigationSarRecordEntity.setId(old.getId());
        }
        irrigationSarRecordEntity.setIrrigationDate(irrigationDate);
        // 设置添加用户
        irrigationSarRecordEntity.setAddUserName(userName);
        // 设置系统时间
        irrigationSarRecordEntity.setAddTime(getNowTime());
        irrigationSarRecordDao.save(irrigationSarRecordEntity);
    }

    /**
     * 删除灌溉记录
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "IrrigationSarRecordServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        irrigationSarRecordDao.deleteById(id);
    }

}
