package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.SysRoleDao;
import com.xkrs.microservice.dao.SysUserDao;
import com.xkrs.microservice.model.entity.SysRoleEntity;
import com.xkrs.microservice.model.entity.SysUserEntity;
import com.xkrs.microservice.model.qo.SysUserQo;
import com.xkrs.microservice.model.vo.SysUserVo;
import com.xkrs.microservice.service.SysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.crypto.keygen.KeyGenerators;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

import java.util.List;
import java.util.regex.Pattern;

import static com.xkrs.microservice.util.DateTimeUtil.*;
import static com.xkrs.microservice.util.EncryptDecryptUtil.encrypt256;

/**
 * 系统用户服务实现
 * @author tajochen
 */
@Service
public class SysUserServiceImpl implements SysUserService {

    Logger logger = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private SysRoleDao sysRoleDao;

    /**
     * 获取指定用户Vo
     * @param userName 用户名
     * @return
     */
    @Override
    public SysUserVo getSysUserVoByUserName(String userName){
        return sysUserDao.selectSysUserByUserName(userName);
    }

    /**
     * 用户头像更新
     * @param userName 用户名
     * @param portraitId 编号
     * @return
     */
    @Override
    public int updateSysUserPortrait(String userName,Integer portraitId){
        SysUserEntity sysUserEntity = sysUserDao.findByUserName(userName);
        if(sysUserEntity==null){
            return 1;
        }
        sysUserEntity.setPortraitId(portraitId);
        sysUserDao.save(sysUserEntity);
        return 0;
    }

    /**
     * 用户密码更新
     * @param userName 用户名
     * @param oldPassword 旧密码
     * @param newPassword 新密码
     * @return
     */
    @Override
    public int updateSysUserPassword(String userName,String oldPassword,String newPassword){
        // 检查旧密码是否正确
        SysUserEntity sysUserEntity = sysUserDao.findByUserName(userName);
        if(sysUserEntity==null){
            return 1;
        }
        String oldPasswordStr = encrypt256(oldPassword + sysUserEntity.getSalt());
        // 如果旧密码不正确
//        if(!oldPasswordStr.equals(sysUserEntity.getPassword())){
//            return  2;
//        }
        String pattern = "^(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{6,16}$";
        boolean isMatch = Pattern.matches(pattern, newPassword);
        // 如果新密码不符合规则
//        if(!isMatch){
//            return 3;
//        }
        String salt = KeyGenerators.string().generateKey();
        sysUserEntity.setSalt(salt);
        sysUserEntity.setPassword(encrypt256(newPassword + salt));
        sysUserDao.save(sysUserEntity);
        return 0;
    }

    /**
     * 检查用户名是否存在
     * @param userName 用户名
     * @return
     */
    @Override
    public boolean checkUserName(String userName) {
        int num = sysUserDao.checkUserName(userName);
        return (num == 0);
    }

    /**
     * 禁用用户账号
     * @param userName
     * @return
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public int disableSysUser(String userName){
        SysUserEntity sysUserEntity = sysUserDao.findByUserName(userName);
        sysUserEntity.setStatusCode(2);
        sysUserDao.save(sysUserEntity);
        return 1;
    }

    /**
     * 启用用户账号
     * @param userName
     * @return
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public int enableSysUser(String userName){
        SysUserEntity sysUserEntity = sysUserDao.findByUserName(userName);
        sysUserEntity.setStatusCode(0);
        sysUserDao.save(sysUserEntity);
        return 1;
    }

    /**
     * 新增用户
     * @param sysUserQo
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public void addUser(SysUserQo sysUserQo) {
        String salt = KeyGenerators.string().generateKey();
        SysUserEntity sysUserEntity = new SysUserEntity();
        sysUserEntity.setUserName(sysUserQo.getUserName());
        sysUserEntity.setSalt(salt);
        sysUserEntity.setPassword(encrypt256(sysUserQo.getPassword() + salt));
        sysUserEntity.setAdmCode(sysUserQo.getAdmCode());
        sysUserEntity.setAddress(sysUserQo.getAddress());
        sysUserEntity.setEmail(sysUserQo.getEmail());
        sysUserEntity.setTelephone(sysUserQo.getTelephone());
        sysUserEntity.setPortraitId(0);
        if(sysUserQo.getBirthdayTs()!=null){
            sysUserEntity.setBirthday(timeMillisToDate(sysUserQo.getBirthdayTs()));
        }
        sysUserEntity.setActiveFlag(0);
        sysUserEntity.setStatusCode(2);
        sysUserEntity.setAddTime(getNowTime());
        sysUserEntity.setDeleteFlag(0);
        sysUserEntity.setMemo(sysUserQo.getMemo());
        sysUserEntity.setRealName(sysUserQo.getRealName());
        sysUserDao.save(sysUserEntity);
        sysRoleDao.insertRelUserRole(sysUserEntity.getUserName(),"role_general_user");
    }

    /**
     * 查询所有用户Vo
     * @return
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public Iterable<SysUserVo> getAllSysUser() {
        Iterable<SysUserVo> ls = sysUserDao.selectAllSysUser();
        return ls;
    }

    @Transactional(rollbackFor=Exception.class)
    @Override
    public SysUserEntity getSysUserByUserName(String userName) {
        return sysUserDao.selectByUserName(userName);
    }

    @Transactional(rollbackFor=Exception.class)
    @Override
    public int updateSysUserLogin(String userName,String ipAddress) {
        return  sysUserDao.updateSysUserLogin(userName,ipAddress);
    }

    /**
     * 更新用户
     * @param sysUserQo
     * @return
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public int updateSysUser(SysUserQo sysUserQo) {
        SysUserEntity sysUserEntity = new SysUserEntity();
        sysUserEntity.setNickName(sysUserQo.getNickName());
        sysUserDao.save(sysUserEntity);
        return  1;
    }

    /**
     * 软删除普通用户
     * @param id 编号
     * @return
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public int softDeleteGeneralUser(Integer id) {
        String adminRole = "role_administor";
        String sysRole ="role_system_manager";

        List<SysRoleEntity> list =  sysRoleDao.selectByUserId(id);
        if(list.size()>0){
            SysRoleEntity sysRoleEntity = list.get(1);
            if(sysRole.equals(sysRoleEntity.getRoleName())||adminRole.equals(sysRoleEntity.getRoleName())){
                sysUserDao.softDeleteGeneralUserById(id);
                return 0;
            }
        }
        return 1;
    }

}
