package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.TifProductDao;
import com.xkrs.microservice.model.entity.TifProductEntity;
import com.xkrs.microservice.model.qo.TifProductQo;
import com.xkrs.microservice.service.TifProductService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;

/**
 * 栅格产品管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "TifProductServiceCache")
public class TifProductServiceImpl implements TifProductService {

    Logger logger = LoggerFactory.getLogger(TifProductServiceImpl.class);

    @Resource
    private TifProductDao tifProductDao;

    /**
     * 获取所有栅格产品
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TifProductEntity> getAllFile(){
        return tifProductDao.findAll();
    }

    /**
     * 获取指定产品编号的产品
     * @param productCode 产品编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public TifProductEntity getByProductCode(String productCode){
        return tifProductDao.findByProductCode(productCode);
    }

    /**
     * 获取指定id的产品
     * @param id 产品编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<TifProductEntity> getById(Integer id){
        return tifProductDao.findById(id);
    }

    /**
     * 获取指定类型的所有产品
     * @param productType 产品类型
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TifProductEntity> getByProductType(String productType){
        return tifProductDao.findByProductType(productType);
    }

    /**
     * 获取指定类型和时间段的所有产品
     * @param productType 产品类型
     * @param startDate 开始日期
     * @param endDate 结束日期
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TifProductEntity> getByProductTypeAndDate(String productType, LocalDate startDate, LocalDate endDate){
        return tifProductDao.findByProductTypeAndProductDateBetween(productType,startDate,endDate);
    }

    /**
     * 获取指定行政区划和时间段的所有产品
     * @param admCode 行政区划代码
     * @param startDate 开始日期
     * @param endDate 结束日期
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TifProductEntity> getByAdmCodeAndProductDateBetween(Integer admCode, LocalDate startDate,LocalDate endDate){
        return tifProductDao.findByAdmCodeAndProductDateBetween(admCode,startDate,endDate);
    }

    /**
     * 获取指定类型和行政区划的所有产品
     * @param admCode 行政区划
     * @param productType 产品类型
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TifProductEntity> getByAdmCodeAndProductType(Integer admCode,String productType){
        return tifProductDao.findByProductTypeAndAdmCode(productType,admCode);
    }

    /**
     * 获取指定类型和行政区划和时间段的所有产品
     * @param admCode 行政区划
     * @param productType 产品类型
     * @param startDate 开始日期
     * @param endDate 结束日期
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TifProductEntity> getByAdmCodeAndProductTypeAndDate(Integer admCode,String productType, LocalDate startDate,LocalDate endDate){
        return tifProductDao.findByProductTypeAndAdmCodeAndProductDateBetween(productType,admCode,startDate,endDate);
    }

    /**
     * 新增栅格产品记录
     * @param tifProductQo
     * @param userName 用户名
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TifProductServiceCache",allEntries = true)
    @Override
    public void add(TifProductQo tifProductQo, String userName){
        TifProductEntity tifProductEntity = new TifProductEntity();
        CopyPropertiesUtil.copy(tifProductQo, tifProductEntity);
        LocalDate productDate = timeMillisToDate(tifProductQo.getProductDateTs());
        tifProductEntity.setProductDate(productDate);
        // 设置添加用户
        tifProductEntity.setAddUserName(userName);
        // 设置系统时间
        tifProductEntity.setCreateTime(getNowTime());
        tifProductDao.save(tifProductEntity);
    }

    /**
     * 删除栅格产品
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TifProductServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        tifProductDao.deleteById(id);
    }

}
