package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.SysRoleDao;
import com.xkrs.microservice.dao.SysUserDao;
import com.xkrs.microservice.dao.TypicalFarmerDao;
import com.xkrs.microservice.model.entity.SysUserEntity;
import com.xkrs.microservice.model.entity.TypicalFarmerEntity;
import com.xkrs.microservice.model.qo.TypicalFarmerQo;
import com.xkrs.microservice.service.TypicalFarmerService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Sort;
import org.springframework.security.crypto.keygen.KeyGenerators;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;
import static com.xkrs.microservice.util.EncryptDecryptUtil.encrypt256;

/**
 * TypicalFarmerService 管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "TypicalFarmerServiceCache")
public class TypicalFarmerServiceImpl implements TypicalFarmerService {

    Logger logger = LoggerFactory.getLogger(TypicalFarmerServiceImpl.class);

    @Resource
    private TypicalFarmerDao typicalFarmerDao;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private SysRoleDao sysRoleDao;

    /**
     * 获取所有农户记录列表
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalFarmerEntity> getAllRecord(){
        return typicalFarmerDao.findAll(Sort.by(Sort.Direction.DESC, "createTime"));
    }

    /**
     * 获取指定id记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional getById(Integer id){
        return typicalFarmerDao.findById(id);
    }

    /**
     * 获取指定乡镇代码的所有农户记录列表
     * @param xzdm 乡镇代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalFarmerEntity> getByTownCode(String xzdm){
        return  typicalFarmerDao.findAllByXzdm(xzdm);
    }

    /**
     * 获取指定区县代码的所有农户记录列表
     * @param qxdm 区县代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalFarmerEntity> getByCountyCode(Integer qxdm){
        return  typicalFarmerDao.findAllByQxdm(qxdm);
    }

    /**
     * 获取指定姓名的所有农户记录列表
     * @param lxr 联系人
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalFarmerEntity> getByLxr(String lxr){
        return typicalFarmerDao.findAllByLxr(lxr);
    }

    /**
     * 获取指定手机号的农户记录列表
     * @param tel 手机号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<TypicalFarmerEntity> getByTel(String tel){
        return typicalFarmerDao.findByTel(tel);
    }

    /**
     * 新增农户记录
     * @param typicalFarmerQo
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalFarmerServiceCache",allEntries = true)
    @Override
    public void add(TypicalFarmerQo typicalFarmerQo, String userName){
        String salt = KeyGenerators.string().generateKey();
        SysUserEntity sysUserEntity = new SysUserEntity();
        sysUserEntity.setUserName(userName);
        sysUserEntity.setSalt(salt);
        sysUserEntity.setPassword(encrypt256(typicalFarmerQo.getPassword() + salt));
        sysUserEntity.setAdmCode(typicalFarmerQo.getQxdm());
        sysUserEntity.setAddress(typicalFarmerQo.getAddress());
        sysUserEntity.setTelephone(typicalFarmerQo.getTel());
        sysUserEntity.setPortraitId(0);
        sysUserEntity.setActiveFlag(0);
        sysUserEntity.setStatusCode(2);
        sysUserEntity.setAddTime(getNowTime());
        sysUserEntity.setDeleteFlag(0);
        sysUserEntity.setRealName(typicalFarmerQo.getLxr());
        sysUserEntity.setMemo("农户");
        sysUserDao.save(sysUserEntity);
        sysRoleDao.insertRelUserRole(sysUserEntity.getUserName(),"role_farm_user");
        TypicalFarmerEntity typicalFarmerEntity = new TypicalFarmerEntity();
        CopyPropertiesUtil.copy( typicalFarmerQo, typicalFarmerEntity);
        typicalFarmerEntity.setAddUserName(userName);
        // 设置协议时间
        LocalDate sDate = timeMillisToDate(typicalFarmerQo.getStrdateTs());
        typicalFarmerEntity.setStrdate(sDate);
        LocalDate eDate = timeMillisToDate(typicalFarmerQo.getEnddateTs());
        typicalFarmerEntity.setEnddate(eDate);
        // 设置系统时间
        typicalFarmerEntity.setCreateTime(getNowTime());
        typicalFarmerDao.save(typicalFarmerEntity);
    }

    /**
     * 修改农户记录
     * @param typicalFarmerQo
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalFarmerServiceCache",allEntries = true)
    @Override
    public void update(TypicalFarmerQo typicalFarmerQo){
        Optional<TypicalFarmerEntity> typicalFarmerEntityOptional = typicalFarmerDao.findByTel(typicalFarmerQo.getTel());
        TypicalFarmerEntity typicalFarmerEntity = typicalFarmerEntityOptional.get();
        typicalFarmerEntity.setLxr(typicalFarmerQo.getLxr());
        typicalFarmerEntity.setLxrid(typicalFarmerQo.getLxrid());
        typicalFarmerEntity.setAddress(typicalFarmerQo.getAddress());
        LocalDate sDate = timeMillisToDate(typicalFarmerQo.getStrdateTs());
        typicalFarmerEntity.setStrdate(sDate);
        LocalDate eDate = timeMillisToDate(typicalFarmerQo.getEnddateTs());
        typicalFarmerEntity.setEnddate(eDate);
        typicalFarmerEntity.setXyzp(typicalFarmerQo.getXyzp());
        typicalFarmerEntity.setRemark(typicalFarmerQo.getRemark());
        typicalFarmerDao.save(typicalFarmerEntity);
    }

    /**
     * 删除农户记录列表
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalFarmerServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        typicalFarmerDao.deleteById(id);
    }

}
