package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.TypicalPlotIrrigationDao;
import com.xkrs.microservice.model.entity.TypicalPlotIrrigationEntity;
import com.xkrs.microservice.model.qo.TypicalPlotIrrigationQo;
import com.xkrs.microservice.service.TypicalPlotIrrigationService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.*;

/**
 * TypicalPlotService 地块灌溉信息管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "TypicalPlotIrrigationService")
public class TypicalPlotIrrigationServiceImpl implements TypicalPlotIrrigationService {

    Logger logger = LoggerFactory.getLogger(TypicalPlotIrrigationServiceImpl.class);

    @Resource
    private TypicalPlotIrrigationDao typicalPlotIrrigationDao;

    /**
     * 获取所有记录列表
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotIrrigationEntity> getAllRecord(){
        return typicalPlotIrrigationDao.findAll(Sort.by(Sort.Direction.DESC, "createTime"));
    }

    /**
     * 获取指定id记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional getById(Integer id){
        return typicalPlotIrrigationDao.findById(id);
    }

    /**
     * 获取指定灌溉编号的地块灌溉信息
     * @param ggbh 灌溉编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<TypicalPlotIrrigationEntity> getByGgbh(String ggbh){
        return typicalPlotIrrigationDao.findByGgbh(ggbh);
    }

    /**
     * 查找灌溉记录列表根据区县编号
     * @param qxbh 区县编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotIrrigationEntity> getByQxbh(Integer qxbh){
        return typicalPlotIrrigationDao.findAllByQxbh(qxbh);
    }

    /**
     * 获取指定编号的地块灌溉信息
     * @param dkbh 地块编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotIrrigationEntity> getByDkbh(String dkbh){
        return typicalPlotIrrigationDao.findAllByDkbh(dkbh);
    }

    /**
     * 查找灌溉记录根据灌溉编号
     * @param qxbh 区县代码
     * @param sTime 开始时间
     * @param eTime 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotIrrigationEntity> findAllByQxbhAndStrtimeBetween(Integer qxbh, LocalDateTime sTime, LocalDateTime eTime){
        return typicalPlotIrrigationDao.findAllByQxbhAndEndtimeBetween(qxbh,sTime,eTime);
    }

    /**
     * 查找灌溉记录根据动态多条件
     * @param xzbh 乡镇街道代码
     * @param cropCode 作物代码
     * @param sTimeTs 开始时间
     * @param eTimeTs 结束时间
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public List<TypicalPlotIrrigationEntity> getAllByCondition(String xzbh, Integer cropCode, Long sTimeTs, Long eTimeTs){
        LocalDateTime startTime = timeMillisToTime(sTimeTs);
        LocalDateTime endTime = timeMillisToTime(eTimeTs);
        return typicalPlotIrrigationDao.getAllByCondition(xzbh,cropCode,startTime,endTime);
    }

    /**
     * 获取指定用户名的所有地块灌溉信息
     * @param userName 用户名
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotIrrigationEntity> getByUserName(String userName){
        return typicalPlotIrrigationDao.findAllByAddUserName(userName);
    }

    /**
     * 新增典型地块灌溉信息
     * @param typicalPlotIrrigationQo
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalPlotIrrigationService",allEntries = true)
    @Override
    public void add(TypicalPlotIrrigationQo typicalPlotIrrigationQo,String userName){
        TypicalPlotIrrigationEntity typicalPlotEntity = new TypicalPlotIrrigationEntity();
        CopyPropertiesUtil.copy( typicalPlotIrrigationQo, typicalPlotEntity);
        typicalPlotEntity.setAddUserName(userName);
        // 设置灌溉时间
        LocalDateTime sTime = timeMillisToTime(typicalPlotIrrigationQo.getStrtimeTs());
        typicalPlotEntity.setStrtime(sTime);
        LocalDateTime eTime = timeMillisToTime(typicalPlotIrrigationQo.getEndtimeTs());
        typicalPlotEntity.setEndtime(eTime);
        // 设置系统时间
        typicalPlotEntity.setCreateTime(getNowTime());
        typicalPlotIrrigationDao.save(typicalPlotEntity);
    }

    /**
     * 删除地块灌溉信息
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalPlotIrrigationService",allEntries = true)
    @Override
    public void delete(Integer id){
        typicalPlotIrrigationDao.deleteById(id);
    }

}
