package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.TypicalPlotDao;
import com.xkrs.microservice.model.entity.TypicalPlotEntity;
import com.xkrs.microservice.model.qo.TypicalPlotQo;
import com.xkrs.microservice.service.TypicalPlotService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.constraints.NotEmpty;
import java.util.Objects;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;
import static com.xkrs.microservice.util.OpenGeoUtil.wktStrToGeom;

/**
 * TypicalPlotService管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "TypicalPlotServiceCache")
public class TypicalPlotServiceImpl implements TypicalPlotService {

    Logger logger = LoggerFactory.getLogger(TypicalPlotServiceImpl.class);

    @Resource
    private TypicalPlotDao typicalPlotDao;

    /**
     * 获取所有记录列表
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotEntity> getAllRecord(){
        return typicalPlotDao.findAll(Sort.by(Sort.Direction.DESC, "createTime"));
    }

    /**
     * 获取指定id记录
     * @param id
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional getById(Integer id){
        return typicalPlotDao.findById(id);
    }

    /**
     * 获取指定编号的典型地块记录
     * @param dkbh 地块编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<TypicalPlotEntity> getByDkbh(String dkbh){
        return typicalPlotDao.findAllByDkbh(dkbh);
    }

    /**
     * 获取指定区县代码的所有典型地块记录
     * @param qxdm 区县代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotEntity> getByCountyCode(Integer qxdm){
        return  typicalPlotDao.findAllByQxdm(qxdm);
    }

    /**
     * 获取指定乡镇街道代码的所有典型地块记录
     * @param xzdm 街道代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotEntity> getByTownCode(String xzdm){
        return typicalPlotDao.findAllByXzdm(xzdm);
    }

    /**
     * 获取指定联系人的所有典型地块记录
     * @param lxr 联系人
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotEntity> getByLxr(String lxr){
        return typicalPlotDao.findAllByLxr(lxr);
    }

    /**
     * 获取指定联系人的所有典型地块记录
     * @param tel 手机号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<TypicalPlotEntity> getByTel(String tel){
        return typicalPlotDao.findAllByTel(tel);
    }

    /**
     * 新增典型地块记录
     * @param typicalPlotQo
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalPlotServiceCache",allEntries = true)
    @Override
    public void add(TypicalPlotQo typicalPlotQo,String userName){
        TypicalPlotEntity typicalPlotEntity = new TypicalPlotEntity();
        CopyPropertiesUtil.copy( typicalPlotQo, typicalPlotEntity);
        typicalPlotEntity.setAddUserName(userName);
        if( !typicalPlotQo.getGeomStr().isEmpty()){
            // 转化geometry
            typicalPlotEntity.setGeom(wktStrToGeom(typicalPlotQo.getGeomStr()));
        }
        // 设置系统时间
        typicalPlotEntity.setCreateTime(getNowTime());
        typicalPlotDao.save(typicalPlotEntity);
    }

    /**
     * 更新典型地块记录
     * @param typicalPlotQo
     * @param userName
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalPlotServiceCache",allEntries = true)
    @Override
    public void update(TypicalPlotQo typicalPlotQo,String userName){
        TypicalPlotEntity typicalPlotEntity = new TypicalPlotEntity();
        CopyPropertiesUtil.copy( typicalPlotQo, typicalPlotEntity);
        typicalPlotEntity.setAddUserName(userName);
        // 转化geometry
        typicalPlotEntity.setGeom(wktStrToGeom(typicalPlotQo.getGeomStr()));
        // 设置系统时间
        typicalPlotEntity.setCreateTime(getNowTime());
        typicalPlotDao.save(typicalPlotEntity);
    }

    /**
     * 删除典型地块记录
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "TypicalPlotServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        typicalPlotDao.deleteById(id);
    }

}
