package com.xkrs.microservice.service.impl;

import com.xkrs.microservice.dao.WcsRecordDao;
import com.xkrs.microservice.model.entity.WcsRecordEntity;
import com.xkrs.microservice.model.qo.WcsRecordQo;
import com.xkrs.microservice.service.WcsRecordService;
import com.xkrs.microservice.util.CopyPropertiesUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.util.Optional;

import static com.xkrs.microservice.util.DateTimeUtil.getNowTime;
import static com.xkrs.microservice.util.DateTimeUtil.timeMillisToDate;

/**
 * 栅格服务管理接口
 * @author tajochen
 */
@Service
@CacheConfig(cacheNames = "WcsRecordServiceCache")
public class WcsRecordServiceImpl implements WcsRecordService {

    Logger logger = LoggerFactory.getLogger(WcsRecordServiceImpl.class);

    @Resource
    private WcsRecordDao wcsRecordDao;

    /**
     * 获取所有栅格服务
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<WcsRecordEntity> getAllService(){
        return wcsRecordDao.findAll();
    }

    /**
     * 获取指定服务编号的栅格服务
     * @param serviceCode 服务代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public WcsRecordEntity getByServiceCode(String serviceCode){
        return wcsRecordDao.findByServiceCode(serviceCode);
    }

    /**
     * 获取指定id的产品
     * @param id 产品编号
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Optional<WcsRecordEntity> getById(Integer id){
        return wcsRecordDao.findById(id);
    }

    /**
     * 获取指定类型的所有栅格服务
     * @param productType 产品类型
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<WcsRecordEntity> getByProductType(String productType){
        return wcsRecordDao.findByProductType(productType);
    }

    /**
     * 获取指定类型和行政区划的所有服务
     * @param admCode 行政区划
     * @param productType 产品类型
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<WcsRecordEntity> getByAdmCodeAndProductType(Integer admCode,String productType) {
        return wcsRecordDao.findByProductTypeAndAdmCode(productType,admCode);
    }

    /**
     * 获取指定类型和时间段的所有栅格服务
     * @param productType 产品类型
     * @param startDate 开始日期
     * @param endDate 结束日期
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<WcsRecordEntity> getByProductTypeAndDate(String productType, LocalDate startDate, LocalDate endDate){
        return wcsRecordDao.findByProductTypeAndProductDateBetween(productType,startDate,endDate);
    }

    /**
     * 获取指定行政区划和时间段的所有服务
     * @param admCode 行政区划代码
     * @param startDate 开始日期
     * @param endDate 行政区划代码
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<WcsRecordEntity> getByAdmCodeAndDate(Integer admCode, LocalDate startDate,LocalDate endDate){
        return wcsRecordDao.findByAdmCodeAndProductDateBetween(admCode,startDate,endDate);
    }

    /**
     * 获取指定类型和行政区划和时间段的所有服务
     * @param admCode 行政区划
     * @param productType 产品类型
     * @param startDate 开始日期
     * @param endDate 结束日期
     * @return
     */
    @Cacheable(keyGenerator = "keyGenerator", unless="#result == null")
    @Override
    public Iterable<WcsRecordEntity> getByAdmCodeAndProductTypeAndDate(Integer admCode,String productType, LocalDate startDate,LocalDate endDate){
        return wcsRecordDao.findByProductTypeAndAdmCodeAndProductDateBetween (productType,admCode,startDate,endDate);
    }

    /**
     * 新增栅格服务记录
     * @param wcsRecordQo
     * @param userName 用户名
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "WcsRecordServiceCache",allEntries = true)
    @Override
    public void add(WcsRecordQo wcsRecordQo, String userName){
        WcsRecordEntity wcsRecordEntity = new WcsRecordEntity();
        CopyPropertiesUtil.copy(wcsRecordQo, wcsRecordEntity);
        LocalDate productDate = timeMillisToDate(wcsRecordQo.getProductDateTs());
        wcsRecordEntity.setProductDate(productDate);
        // 设置添加用户
        wcsRecordEntity.setAddUserName(userName);
        // 设置系统时间
        wcsRecordEntity.setCreateTime(getNowTime());
        wcsRecordDao.save(wcsRecordEntity);
    }

    /**
     * 删除栅格服务
     * @param id
     */
    @Transactional(rollbackFor=Exception.class)
    @CacheEvict(value = "WcsRecordServiceCache",allEntries = true)
    @Override
    public void delete(Integer id){
        wcsRecordDao.deleteById(id);
    }

}
